import { BN, BorshAccountsCoder, Idl } from "@coral-xyz/anchor";
import { Staking } from "../artifacts/staking";
import * as IDL from "../artifacts/staking.json";
import { PublicKey } from "@solana/web3.js";

export type StakingType = Staking;
export const StakingIDL = IDL as Idl;
export const accountsCoder = new BorshAccountsCoder(StakingIDL);

export enum AccountName {
  FlexibleVault = "FlexibleVault",
  LockupVault = "LockupVault",
  LockupStakingAccount = "LockupStakingAccount",
}

export type FlexibleVaultData = {
  bump: number;
  totalSupply: BN;
  rewardPerTokenStored: BN;
  rewardRate: BN;
  lastUpdateTime: BN;
  withdrawalDelayTime: BN;
  totalRewardClaimed: BN;
}

export type LockupVaultData = {
  bump: number;
  index: BN;
  apy: BN;
  lockupDuration: BN;
  minStakeAmount: BN;
  totalRewardClaimed: BN;
}

export type LockupStakingAccountData = {
  bump: number;
  index: BN;
  owner: PublicKey;
  lockupVault: PublicKey;
  committedApy: BN;
  lockedAmount: BN;
  lockedAt: BN;
  unlocked: boolean;
}

export type WithdrawPositionInfo = {
  index: number;
  amount: string;
  startDelayTime: string;
  unlockTime: string;
  claimed: boolean;
}

export type FlexibleStakingInfo = {
  totalStakedAmount: string;
  currentApr: number;
  pendingReward: string;
  withdrawPositions: WithdrawPositionInfo[];
}
